@@ -15,7 +15,11 @@
 """Chex: Testing made fun, in JAX!"""
 
 from chex._src.asserts import assert_axis_dimension
+from chex._src.asserts import assert_axis_dimension_comparator
 from chex._src.asserts import assert_axis_dimension_gt
+from chex._src.asserts import assert_axis_dimension_gteq
+from chex._src.asserts import assert_axis_dimension_lt
+from chex._src.asserts import assert_axis_dimension_lteq
 from chex._src.asserts import assert_devices_available
 from chex._src.asserts import assert_equal
 from chex._src.asserts import assert_equal_rank
@@ -42,14 +46,23 @@ from chex._src.asserts import assert_tree_all_equal_comparator  # Deprecated
 from chex._src.asserts import assert_tree_all_equal_shapes  # Deprecated
 from chex._src.asserts import assert_tree_all_equal_structs  # Deprecated
 from chex._src.asserts import assert_tree_all_finite
+from chex._src.asserts import assert_tree_has_only_ndarrays
+from chex._src.asserts import assert_tree_is_on_device
+from chex._src.asserts import assert_tree_is_on_host
+from chex._src.asserts import assert_tree_is_sharded
 from chex._src.asserts import assert_tree_no_nones
 from chex._src.asserts import assert_tree_shape_prefix
+from chex._src.asserts import assert_tree_shape_suffix
 from chex._src.asserts import assert_trees_all_close
+from chex._src.asserts import assert_trees_all_equal
 from chex._src.asserts import assert_trees_all_equal_comparator
+from chex._src.asserts import assert_trees_all_equal_dtypes
 from chex._src.asserts import assert_trees_all_equal_shapes
 from chex._src.asserts import assert_trees_all_equal_structs
 from chex._src.asserts import assert_type
 from chex._src.asserts import clear_trace_counter
+from chex._src.asserts import disable_asserts
+from chex._src.asserts import enable_asserts
 from chex._src.asserts import if_args_not_none
 from chex._src.dataclass import dataclass
 from chex._src.dataclass import mappable_dataclass
@@ -60,17 +73,17 @@ from chex._src.fake import set_n_cpu_devices
 from chex._src.pytypes import Array
 from chex._src.pytypes import ArrayBatched
 from chex._src.pytypes import ArrayDevice
+from chex._src.pytypes import ArrayDType
 from chex._src.pytypes import ArrayNumpy
 from chex._src.pytypes import ArraySharded
 from chex._src.pytypes import ArrayTree
-from chex._src.pytypes import CpuDevice
 from chex._src.pytypes import Device
-from chex._src.pytypes import GpuDevice
 from chex._src.pytypes import Numeric
 from chex._src.pytypes import PRNGKey
+from chex._src.pytypes import PyTreeDef
 from chex._src.pytypes import Scalar
 from chex._src.pytypes import Shape
-from chex._src.pytypes import TpuDevice
+from chex._src.restrict_backends import restrict_backends
 from chex._src.variants import all_variants
 from chex._src.variants import ChexVariantType
 from chex._src.variants import params_product
@@ -78,18 +91,23 @@ from chex._src.variants import TestCase
 from chex._src.variants import variants
 
 
-__version__ = "0.0.8"
+__version__ = "0.1.3"
 
 __all__ = (
     "all_variants",
     "Array",
     "ArrayBatched",
     "ArrayDevice",
+    "ArrayDType",
     "ArrayNumpy",
     "ArraySharded",
     "ArrayTree",
     "assert_axis_dimension",
+    "assert_axis_dimension_comparator",
     "assert_axis_dimension_gt",
+    "assert_axis_dimension_gteq",
+    "assert_axis_dimension_lt",
+    "assert_axis_dimension_lteq",
     "assert_devices_available",
     "assert_equal",
     "assert_equal_rank",
@@ -116,32 +134,40 @@ __all__ = (
     "assert_tree_all_equal_shapes",  # Deprecated
     "assert_tree_all_equal_structs",  # Deprecated
     "assert_tree_all_finite",
+    "assert_tree_has_only_ndarrays",
+    "assert_tree_is_on_device",
+    "assert_tree_is_on_host",
+    "assert_tree_is_sharded",
     "assert_tree_no_nones",
     "assert_tree_shape_prefix",
+    "assert_tree_shape_suffix",
     "assert_trees_all_close",
+    "assert_trees_all_equal",
     "assert_trees_all_equal_comparator",
+    "assert_trees_all_equal_dtypes",
     "assert_trees_all_equal_shapes",
     "assert_trees_all_equal_structs",
     "assert_type",
     "ChexVariantType",
     "clear_trace_counter",
-    "CpuDevice",
     "dataclass",
     "Device",
+    "disable_asserts",
+    "enable_asserts",
     "fake_jit",
     "fake_pmap",
     "fake_pmap_and_jit",
-    "GpuDevice",
     "if_args_not_none",
     "mappable_dataclass",
     "Numeric",
     "params_product",
     "PRNGKey",
+    "PyTreeDef",
+    "restrict_backends",
     "Scalar",
     "set_n_cpu_devices",
     "Shape",
     "TestCase",
-    "TpuDevice",
     "variants",
 )
 
