@@ -15,6 +15,7 @@
 """Tests for `asserts.py`."""
 
 import functools
+import re
 
 from absl.testing import absltest
 from absl.testing import parameterized
@@ -26,6 +27,7 @@ import jax.numpy as jnp
 import numpy as np
 
 _get_err_regex = asserts_internal.get_err_regex
+_num_devices_available = asserts_internal.num_devices_available
 
 
 def as_arrays(arrays):
@@ -40,6 +42,27 @@ def emplace(arrays):
   return arrays
 
 
+class AssertsSwitchTest(parameterized.TestCase):
+  """Tests for enable/disable_asserts."""
+
+  def test_enable_disable_asserts(self):
+    with self.assertRaisesRegex(AssertionError, _get_err_regex('scalar')):
+      asserts.assert_scalar('test')
+
+    asserts.disable_asserts()
+    asserts.assert_scalar('test')
+
+    asserts.enable_asserts()
+    with self.assertRaisesRegex(AssertionError, _get_err_regex('scalar')):
+      asserts.assert_scalar('test')
+
+    asserts.disable_asserts()
+    asserts.assert_is_divisible(13, 5)
+
+    # To avoid side effects.
+    asserts.enable_asserts()
+
+
 class AssertMaxTracesTest(variants.TestCase):
 
   def setUp(self):
@@ -229,7 +252,7 @@ class ScalarAssertTest(parameterized.TestCase):
     asserts.assert_scalar_non_negative(0.5)
     asserts.assert_scalar_non_negative(0.)
     with self.assertRaisesRegex(AssertionError,
-                                _get_err_regex('must be non negative')):
+                                _get_err_regex('must be non-negative')):
       asserts.assert_scalar_non_negative(-0.5)
 
   def test_scalar_negative(self):
@@ -297,6 +320,29 @@ class EqualShapeAssertTest(parameterized.TestCase):
                                 _get_err_regex('different shape prefixes')):
       asserts.assert_equal_shape_prefix(arrays, prefix_len=1)
 
+  @parameterized.named_parameters(
+      ('first_dim', [[2, 3], [2, 4], [2, 5]], 0),
+      ('last_dim', [[3, 5, 7], [2, 7], [4, 7]], -1),  # Note different ranks.
+      ('first_few_dims', [[1, 2, 3], [1, 2, 4], [1, 2, 5]], [0, 1]),
+      ('first_and_last', [[1, 2, 1], [1, 3, 1], [1, 4, 1]], [0, 2]),
+      ('first_and_last_neg', [[1, 2, 3, 4], [1, 5, 4], [1, 4]], [0, -1]),
+  )
+  def test_equal_shape_at_dims_should_pass(self, shapes, dims):
+    arrays = [array_from_shape(*shape) for shape in shapes]
+    asserts.assert_equal_shape(arrays, dims=dims)
+
+  @parameterized.named_parameters(
+      ('first_dim', [[1, 2], [2, 2]], 0),
+      ('last_dim', [[1, 3], [1, 4]], 1),
+      ('last_dim_neg', [[1, 3], [1, 4]], -1),
+      ('multiple_dims', [[1, 2, 3], [1, 2, 4]], [0, 2]),
+  )
+  def test_equal_shape_at_dims_should_fail(self, shapes, dims):
+    arrays = [array_from_shape(*shape) for shape in shapes]
+    with self.assertRaisesRegex(
+        AssertionError, _get_err_regex('have different shapes at dims')):
+      asserts.assert_equal_shape(arrays, dims=dims)
+
 
 class ShapeAssertTest(parameterized.TestCase):
 
@@ -578,7 +624,7 @@ class TypeAssertTest(parameterized.TestCase):
     with self.assertRaisesRegex(AssertionError,
                                 _get_err_regex('unsupported dtype')):
       asserts.assert_type([a_float, an_int, a_np_float, a_jax_int],
-                          [np.complex, np.complex, float, int])
+                          [complex, complex, float, int])
 
 
 class AxisDimensionAssertionsTest(parameterized.TestCase):
@@ -626,6 +672,72 @@ class AxisDimensionAssertionsTest(parameterized.TestCase):
                                   _get_err_regex('not available')):
         asserts.assert_axis_dimension_gt(tensor, axis=i, val=0)
 
+  def test_assert_axis_dimension_gteq_pass(self):
+    tensor = jnp.ones((3, 2, 7, 2))
+    for i in range(-tensor.ndim, tensor.ndim):
+      s = tensor.shape[i]
+      asserts.assert_axis_dimension_gteq(tensor, axis=i, val=s)
+
+  def test_assert_axis_dimension_gteq_fail(self):
+    tensor = jnp.ones((3, 2, 7, 2))
+    for i in range(-tensor.ndim, tensor.ndim):
+      s = tensor.shape[i]
+      with self.assertRaisesRegex(
+          AssertionError,
+          _get_err_regex('Expected tensor to have dimension greater than or')):
+        asserts.assert_axis_dimension_gteq(tensor, axis=i, val=s + 1)
+
+  def test_assert_axis_dimension_gteq_axis_invalid(self):
+    tensor = jnp.ones((3, 2))
+    for i in (2, -3):
+      with self.assertRaisesRegex(AssertionError,
+                                  _get_err_regex('not available')):
+        asserts.assert_axis_dimension_gteq(tensor, axis=i, val=0)
+
+  def test_assert_axis_dimension_lt_pass(self):
+    tensor = jnp.ones((3, 2, 7, 2))
+    for i in range(-tensor.ndim, tensor.ndim):
+      s = tensor.shape[i]
+      asserts.assert_axis_dimension_lt(tensor, axis=i, val=s + 1)
+
+  def test_assert_axis_dimension_lt_fail(self):
+    tensor = jnp.ones((3, 2, 7, 2))
+    for i in range(-tensor.ndim, tensor.ndim):
+      s = tensor.shape[i]
+      with self.assertRaisesRegex(
+          AssertionError,
+          _get_err_regex('Expected tensor to have dimension less than')):
+        asserts.assert_axis_dimension_lt(tensor, axis=i, val=s)
+
+  def test_assert_axis_dimension_lt_axis_invalid(self):
+    tensor = jnp.ones((3, 2))
+    for i in (2, -3):
+      with self.assertRaisesRegex(AssertionError,
+                                  _get_err_regex('not available')):
+        asserts.assert_axis_dimension_lt(tensor, axis=i, val=0)
+
+  def test_assert_axis_dimension_lteq_pass(self):
+    tensor = jnp.ones((3, 2, 7, 2))
+    for i in range(-tensor.ndim, tensor.ndim):
+      s = tensor.shape[i]
+      asserts.assert_axis_dimension_lteq(tensor, axis=i, val=s)
+
+  def test_assert_axis_dimension_lteq_fail(self):
+    tensor = jnp.ones((3, 2, 7, 2))
+    for i in range(-tensor.ndim, tensor.ndim):
+      s = tensor.shape[i]
+      with self.assertRaisesRegex(
+          AssertionError,
+          _get_err_regex('Expected tensor to have dimension less than or')):
+        asserts.assert_axis_dimension_lteq(tensor, axis=i, val=s - 1)
+
+  def test_assert_axis_dimension_lteq_axis_invalid(self):
+    tensor = jnp.ones((3, 2))
+    for i in (2, -3):
+      with self.assertRaisesRegex(AssertionError,
+                                  _get_err_regex('not available')):
+        asserts.assert_axis_dimension_lteq(tensor, axis=i, val=0)
+
 
 class TreeAssertionsTest(parameterized.TestCase):
 
@@ -684,6 +796,32 @@ class TreeAssertionsTest(parameterized.TestCase):
         AssertionError, _get_err_regex('Tree contains non-finite value')):
       asserts.assert_tree_all_finite(inf_tree)
 
+  def test_assert_trees_all_equal_passes_same_tree(self):
+    tree1 = {
+        'a': [jnp.zeros((1,))],
+        'b': ([0], (0,), 0),
+    }
+    asserts.assert_trees_all_equal(tree1, tree1)
+
+  def test_assert_trees_all_equal_passes_values_equal(self):
+    tree1 = (jnp.array([0.0, 0.0]),)
+    tree2 = (jnp.array([0.0, 0.0]),)
+    asserts.assert_trees_all_equal(tree1, tree2)
+
+  def test_assert_trees_all_equal_fail_values_close_but_not_equal(self):
+    tree1 = (jnp.array([1.0, 1.0]),)
+    tree2 = (jnp.array([1.0, 1.0 + 5e-7]),)
+    with self.assertRaisesRegex(AssertionError,
+                                _get_err_regex('Values not exactly equal')):
+      asserts.assert_trees_all_equal(tree1, tree2)
+
+  def test_assert_trees_all_equal_nones(self):
+    tree = {'a': [jnp.zeros((1,))], 'b': None}
+    asserts.assert_trees_all_equal(tree, tree, ignore_nones=True)
+    with self.assertRaisesRegex(AssertionError,
+                                _get_err_regex('`None` detected')):
+      asserts.assert_trees_all_equal(tree, tree, ignore_nones=False)
+
   def test_assert_trees_all_close_passes_same_tree(self):
     tree1 = {
         'a': [jnp.zeros((1,))],
@@ -696,10 +834,10 @@ class TreeAssertionsTest(parameterized.TestCase):
     tree2 = (jnp.array([0.0, 0.0]),)
     asserts.assert_trees_all_close(tree1, tree2)
 
-  def test_assert_trees_all_close_passes_values_close(self):
+  def test_assert_trees_all_close_passes_values_close_but_not_equal(self):
     tree1 = (jnp.array([1.0, 1.0]),)
-    tree2 = (jnp.array([1.0, 1.0 + 1e-9]),)
-    asserts.assert_trees_all_close(tree1, tree2)
+    tree2 = (jnp.array([1.0, 1.0 + 5e-7]),)
+    asserts.assert_trees_all_close(tree1, tree2, rtol=1e-6)
 
   def test_assert_trees_all_close_nones(self):
     tree = {'a': [jnp.zeros((1,))], 'b': None}
@@ -708,6 +846,21 @@ class TreeAssertionsTest(parameterized.TestCase):
                                 _get_err_regex('`None` detected')):
       asserts.assert_trees_all_close(tree, tree, ignore_nones=False)
 
+  def test_assert_trees_all_close_bfloat16(self):
+    tree1 = {'a': jnp.asarray([0.8, 1.6], dtype=jnp.bfloat16)}
+    tree2 = {
+        'a': jnp.asarray([0.8, 1.6], dtype=jnp.bfloat16).astype(jnp.float32)
+    }
+    tree3 = {'a': jnp.asarray([0.8, 1.7], dtype=jnp.bfloat16)}
+    asserts.assert_trees_all_close(tree1, tree1)
+    asserts.assert_trees_all_close(tree1, tree2)
+    with self.assertRaisesRegex(
+        AssertionError, _get_err_regex('Values not approximately equal')):
+      asserts.assert_trees_all_close(tree1, tree3)
+    with self.assertRaisesRegex(
+        AssertionError, _get_err_regex('Values not approximately equal')):
+      asserts.assert_trees_all_close(tree2, tree3)
+
   def test_assert_trees_all_equal_shapes_nones(self):
     tree = {'a': [jnp.zeros((1,))], 'b': None}
     asserts.assert_trees_all_equal_shapes(tree, tree, ignore_nones=True)
@@ -715,6 +868,288 @@ class TreeAssertionsTest(parameterized.TestCase):
                                 _get_err_regex('`None` detected')):
       asserts.assert_trees_all_equal_shapes(tree, tree, ignore_nones=False)
 
+  def test_assert_tree_has_only_ndarrays(self):
+    # Check correct inputs.
+    asserts.assert_tree_has_only_ndarrays({'a': jnp.zeros(1), 'b': np.ones(3)})
+    asserts.assert_tree_has_only_ndarrays(np.zeros(4))
+    asserts.assert_tree_has_only_ndarrays(())
+
+    # Check incorrect inputs.
+    with self.assertRaisesRegex(AssertionError,
+                                _get_err_regex('\'b\' is not an ndarray')):
+      asserts.assert_tree_has_only_ndarrays({'a': jnp.zeros((1,)), 'b': 1})
+    with self.assertRaisesRegex(AssertionError,
+                                _get_err_regex('\'b/1\' is not an ndarray')):
+      asserts.assert_tree_has_only_ndarrays({'a': jnp.zeros(101), 'b': [1, 2]})
+
+    # Check `None`.
+    tree_with_none = (np.array([2]), None)
+    asserts.assert_tree_has_only_ndarrays(tree_with_none, ignore_nones=True)
+    with self.assertRaisesRegex(AssertionError,
+                                _get_err_regex('`None` detected')):
+      asserts.assert_tree_has_only_ndarrays(tree_with_none)
+
+  def test_assert_tree_is_on_host(self):
+    cpu = jax.devices('cpu')[0]
+
+    # Check Numpy arrays.
+    for flag in (False, True):
+      asserts.assert_tree_is_on_host({'a': np.zeros(1), 'b': np.ones(3)},
+                                     allow_cpu_device=flag)
+      asserts.assert_tree_is_on_host(np.zeros(4), allow_cpu_device=flag)
+      asserts.assert_tree_is_on_host(
+          jax.device_get(jax.device_put(np.ones(3))), allow_cpu_device=flag)
+      asserts.assert_tree_is_on_host((), allow_cpu_device=flag)
+
+    # Check DeviceArray (for platforms other than CPU).
+    with self.assertRaisesRegex(AssertionError,
+                                _get_err_regex('\'a\' resides on')):
+      asserts.assert_tree_is_on_host({'a': jnp.zeros(1)},
+                                     allow_cpu_device=False)
+    with self.assertRaisesRegex(AssertionError,
+                                _get_err_regex('\'a\' resides on')):
+      asserts.assert_tree_is_on_host({'a': jax.device_put(np.zeros(1))},
+                                     allow_cpu_device=False)
+
+    # Check Jax arrays on CPU.
+    cpu_arr = jax.device_put(np.ones(5), cpu)
+    asserts.assert_tree_is_on_host({'a': cpu_arr})
+    asserts.assert_tree_is_on_host({'a': np.zeros(1), 'b': cpu_arr})
+
+    # Disallow JAX arrays on CPU.
+    with self.assertRaisesRegex(AssertionError,
+                                _get_err_regex('\'a\' resides on.*CPU')):
+      asserts.assert_tree_is_on_host({'a': cpu_arr},
+                                     allow_cpu_device=False)
+
+    with self.assertRaisesRegex(AssertionError,
+                                _get_err_regex('\'b\' resides on.*CPU')):
+      asserts.assert_tree_is_on_host({'a': np.zeros(1), 'b': cpu_arr},
+                                     allow_cpu_device=False)
+
+    # Check incorrect inputs.
+    with self.assertRaisesRegex(AssertionError,
+                                _get_err_regex('\'b\' is not an ndarray')):
+      asserts.assert_tree_is_on_host({'a': np.zeros(1), 'b': 1})
+
+    # ShardedArrays are disallowed.
+    with self.assertRaisesRegex(AssertionError,
+                                _get_err_regex('\'a\' has unexpected type')):
+      asserts.assert_tree_is_on_host(
+          {'a': jax.device_put_replicated(np.zeros(1), (cpu,))},
+          allow_cpu_device=False)
+
+    # Check `None`.
+    tree_with_none = (np.array([2]), None)
+    asserts.assert_tree_is_on_host(tree_with_none, ignore_nones=True)
+    with self.assertRaisesRegex(AssertionError,
+                                _get_err_regex('`None` detected')):
+      asserts.assert_tree_is_on_host(tree_with_none)
+
+  def test_assert_tree_is_on_device(self):
+    # Check CPU platform.
+    cpu = jax.devices('cpu')[0]
+    to_cpu = lambda x: jax.device_put(x, cpu)
+
+    cpu_tree = {'a': to_cpu(np.zeros(1)), 'b': to_cpu(np.ones(3))}
+    asserts.assert_tree_is_on_device(cpu_tree, device=cpu)
+    asserts.assert_tree_is_on_device(cpu_tree, platform='cpu')
+    asserts.assert_tree_is_on_device(cpu_tree, platform=['cpu'])
+    asserts.assert_tree_is_on_device(cpu_tree, device=cpu, platform='')
+
+    with self.assertRaisesRegex(AssertionError,
+                                _get_err_regex('\'a\' resides on \'cpu\'')):
+      asserts.assert_tree_is_on_device(cpu_tree, platform='tpu')
+
+    with self.assertRaisesRegex(AssertionError,
+                                _get_err_regex('\'b\' resides on \'cpu\'')):
+      asserts.assert_tree_is_on_device(cpu_tree, platform=('tpu', 'gpu'))
+
+    # Check TPU platform (if available).
+    if _num_devices_available('tpu') > 1:
+      tpu_1, tpu_2 = jax.devices('tpu')[:2]
+      to_tpu_1 = lambda x: jax.device_put(x, tpu_1)
+      to_tpu_2 = lambda x: jax.device_put(x, tpu_2)
+
+      tpu_1_tree = {'a': to_tpu_1(np.zeros(1)), 'b': to_tpu_1(np.ones(3))}
+      tpu_2_tree = {'a': to_tpu_2(np.zeros(1)), 'b': to_tpu_2(np.ones(3))}
+      tpu_1_2_tree = {'a': to_tpu_1(np.zeros(1)), 'b': to_tpu_2(np.ones(3))}
+
+      # Device asserts.
+      asserts.assert_tree_is_on_device(tpu_1_tree, device=tpu_1)
+      asserts.assert_tree_is_on_device(tpu_2_tree, device=tpu_2)
+
+      with self.assertRaisesRegex(AssertionError,
+                                  _get_err_regex('\'a\' resides on TPU_0')):
+        asserts.assert_tree_is_on_device(tpu_1_tree, device=tpu_2)
+
+      with self.assertRaisesRegex(AssertionError,
+                                  _get_err_regex('\'a\' resides on TPU_1')):
+        asserts.assert_tree_is_on_device(tpu_2_tree, device=tpu_1)
+
+      with self.assertRaisesRegex(AssertionError,
+                                  _get_err_regex('\'a\' resides on .*CPU')):
+        asserts.assert_tree_is_on_device(cpu_tree, device=tpu_2)
+
+      # Platform asserts.
+      asserts.assert_tree_is_on_device(tpu_1_tree, platform='tpu')
+      asserts.assert_tree_is_on_device(tpu_2_tree, platform='tpu')
+      with self.assertRaisesRegex(AssertionError,
+                                  _get_err_regex('\'a\' resides on \'tpu\'')):
+        asserts.assert_tree_is_on_device(tpu_1_tree, platform='cpu')
+
+      with self.assertRaisesRegex(AssertionError,
+                                  _get_err_regex('\'a\' resides on \'tpu\'')):
+        asserts.assert_tree_is_on_device(tpu_2_tree, platform='gpu')
+
+      # Mixed cases.
+      asserts.assert_tree_is_on_device(tpu_1_2_tree, platform='tpu')
+      asserts.assert_tree_is_on_device((tpu_1_2_tree, cpu_tree),
+                                       platform=('cpu', 'tpu'))
+      with self.assertRaisesRegex(AssertionError,
+                                  _get_err_regex('\'1/a\' resides on \'cpu\'')):
+        asserts.assert_tree_is_on_device((tpu_1_2_tree, cpu_tree),
+                                         platform=('tpu'))
+      with self.assertRaisesRegex(AssertionError,
+                                  _get_err_regex('\'0/a\' resides on \'tpu\'')):
+        asserts.assert_tree_is_on_device((tpu_1_2_tree, cpu_tree),
+                                         platform=('cpu', 'gpu'))
+
+    # Check incorrect inputs.
+    with self.assertRaisesRegex(AssertionError,
+                                _get_err_regex('\'b\' is not an ndarray')):
+      asserts.assert_tree_is_on_device({'a': np.zeros(1), 'b': 1})
+
+    with self.assertRaisesRegex(AssertionError,
+                                _get_err_regex('\'b\' has unexpected type')):
+      asserts.assert_tree_is_on_device({'a': jnp.zeros(1), 'b': np.ones(3)})
+
+    with self.assertRaisesRegex(AssertionError,
+                                _get_err_regex('\'a\' is a ShardedDeviceArra')):
+      # ShardedArrays are disallowed.
+      asserts.assert_tree_is_on_device(
+          {'a': jax.device_put_replicated(np.zeros(1), (cpu,))}, device=cpu)
+
+    # Check `None`.
+    asserts.assert_tree_is_on_device((None,), ignore_nones=True)
+    with self.assertRaisesRegex(AssertionError,
+                                _get_err_regex('`None` detected')):
+      asserts.assert_tree_is_on_device((None,))
+
+  def test_assert_tree_is_sharded(self):
+    np_tree = {'a': np.zeros(1), 'b': np.ones(3)}
+
+    def _format(*devs):
+      return re.escape(f'{devs}')
+
+    # Check single-device case.
+    cpu = jax.devices('cpu')[0]
+    cpu_tree = jax.device_put_replicated(np_tree, (cpu,))
+
+    asserts.assert_tree_is_sharded(cpu_tree, devices=(cpu,))
+    asserts.assert_tree_is_sharded((), devices=(cpu,))
+
+    with self.assertRaisesRegex(
+        AssertionError, _get_err_regex(r'\'a\' is sharded.*expected \(\)')):
+      asserts.assert_tree_is_sharded(cpu_tree, devices=())
+
+    # DeviceArrays are disallowed.
+    with self.assertRaisesRegex(
+        AssertionError, _get_err_regex('\'a\' is not a ShardedDeviceArray')):
+      asserts.assert_tree_is_sharded({'a': jax.device_put(np.zeros(1), cpu)},
+                                     devices=(cpu,))
+
+    with self.assertRaisesRegex(
+        AssertionError,
+        _get_err_regex(f'\'a\' is sharded across {_format(cpu)}.*'
+                       f'expected {_format(cpu, cpu)}')):
+      asserts.assert_tree_is_sharded(cpu_tree, devices=(cpu, cpu))
+
+    # Check multiple-devices case (if available).
+    if _num_devices_available('tpu') > 1:
+      tpu_1, tpu_2 = jax.devices('tpu')[:2]
+
+      tpu_1_tree = jax.device_put_replicated(np_tree, (tpu_1,))
+      tpu_2_tree = jax.device_put_replicated(np_tree, (tpu_2,))
+      tpu_1_2_tree = jax.device_put_replicated(np_tree, (tpu_1, tpu_2))
+      tpu_2_1_tree = jax.device_put_replicated(np_tree, (tpu_2, tpu_1))
+
+      asserts.assert_tree_is_sharded(tpu_1_2_tree, devices=(tpu_1, tpu_2))
+      asserts.assert_tree_is_sharded(tpu_2_1_tree, devices=(tpu_2, tpu_1))
+
+      # Wrong device.
+      with self.assertRaisesRegex(
+          AssertionError,
+          _get_err_regex(f'\'a\' is sharded across {_format(tpu_1)}.*'
+                         f'expected {_format(tpu_2)}')):
+        asserts.assert_tree_is_sharded(tpu_1_tree, devices=(tpu_2,))
+
+      with self.assertRaisesRegex(
+          AssertionError,
+          _get_err_regex(f'\'a\' is sharded across {_format(cpu)}.*'
+                         f'expected {_format(tpu_2)}')):
+        asserts.assert_tree_is_sharded(cpu_tree, devices=(tpu_2,))
+
+      # Too many devices.
+      with self.assertRaisesRegex(
+          AssertionError,
+          _get_err_regex(f'\'a\' is sharded across {_format(tpu_1)}.*'
+                         f'expected {_format(tpu_1, tpu_2)}')):
+        asserts.assert_tree_is_sharded(tpu_1_tree, devices=(tpu_1, tpu_2))
+
+      with self.assertRaisesRegex(
+          AssertionError,
+          _get_err_regex(f'\'a\' is sharded across {_format(tpu_1, tpu_2)}.*'
+                         f'expected {_format(tpu_1, tpu_2, cpu)}')):
+        asserts.assert_tree_is_sharded(
+            tpu_1_2_tree, devices=(tpu_1, tpu_2, cpu))
+
+      # Wrong order.
+      with self.assertRaisesRegex(
+          AssertionError,
+          _get_err_regex(f'\'a\' is sharded across {_format(tpu_2, tpu_1)}.*'
+                         f'expected {_format(tpu_1, tpu_2)}')):
+        asserts.assert_tree_is_sharded(tpu_2_1_tree, devices=(tpu_1, tpu_2))
+
+      # Mixed cases.
+      mixed_tree = (tpu_1_tree, tpu_2_tree)
+
+      with self.assertRaisesRegex(
+          AssertionError,
+          _get_err_regex(f'\'0/a\' is sharded across {_format(tpu_1)}.*'
+                         f'expected {_format(tpu_2)}')):
+        asserts.assert_tree_is_sharded(mixed_tree, devices=(tpu_2,))
+      with self.assertRaisesRegex(
+          AssertionError,
+          _get_err_regex(f'\'1/a\' is sharded across {_format(tpu_2)}.*'
+                         f'expected {_format(tpu_1)}')):
+        asserts.assert_tree_is_sharded(mixed_tree, devices=(tpu_1,))
+
+      with self.assertRaisesRegex(
+          AssertionError,
+          _get_err_regex(f'\'0/a\' is sharded across {_format(tpu_1)}.*'
+                         f'expected {_format(tpu_1, tpu_2)}')):
+        asserts.assert_tree_is_sharded(mixed_tree, devices=(tpu_1, tpu_2))
+
+    # Check incorrect inputs.
+    with self.assertRaisesRegex(AssertionError,
+                                _get_err_regex('\'1\' is not an ndarray')):
+      asserts.assert_tree_is_sharded((cpu_tree, 1123), devices=(cpu,))
+
+    with self.assertRaisesRegex(
+        AssertionError, _get_err_regex('\'a\' is not a ShardedDeviceArray')):
+      asserts.assert_tree_is_sharded({'a': jnp.zeros(1)}, devices=(cpu,))
+
+    with self.assertRaisesRegex(
+        AssertionError, _get_err_regex('\'a\' is not a ShardedDeviceArray')):
+      asserts.assert_tree_is_sharded({'a': np.zeros(1)}, devices=(cpu,))
+
+    # Check `None`.
+    asserts.assert_tree_is_sharded((None,), devices=(cpu,), ignore_nones=True)
+    with self.assertRaisesRegex(AssertionError,
+                                _get_err_regex('`None` detected')):
+      asserts.assert_tree_is_sharded((None,), devices=(cpu,))
+
   def test_assert_tree_no_nones(self):
     tree_ok = {'a': [jnp.zeros((1,))], 'b': 1}
     asserts.assert_tree_no_nones(tree_ok)
@@ -787,9 +1222,7 @@ class TreeAssertionsTest(parameterized.TestCase):
 
     with self.assertRaisesRegex(
         AssertionError,
-        _get_err_regex(
-            r'Tree leaf \'x/y\'.*diffent from expected: \(3, 2\) != \(3, 2, 1\)'
-        )):
+        _get_err_regex(r'leaf \'x/y\' has a shape of length 2')):
       asserts.assert_tree_shape_prefix(tree, (3, 2, 1))
 
   def test_assert_tree_shape_prefix_none(self):
@@ -800,6 +1233,106 @@ class TreeAssertionsTest(parameterized.TestCase):
                                 _get_err_regex('`None` detected')):
       asserts.assert_tree_shape_prefix(tree, (3,), ignore_nones=False)
 
+  def test_assert_tree_shape_suffix_matching(self):
+    tree = {'x': {'y': np.zeros([4, 2, 1])}, 'z': np.zeros([2, 1])}
+    asserts.assert_tree_shape_suffix(tree, ())
+    asserts.assert_tree_shape_suffix(tree, (1,))
+    asserts.assert_tree_shape_suffix(tree, (2, 1))
+
+  def test_assert_tree_shape_suffix_mismatch(self):
+    tree = {'x': {'y': np.zeros([4, 2, 1])}, 'z': np.zeros([1, 1])}
+
+    with self.assertRaisesRegex(
+        AssertionError,
+        _get_err_regex(
+            r'Tree leaf \'z\'.*different from expected: \(1, 1\) != \(2, 1\)')):
+      asserts.assert_tree_shape_suffix(tree, (2, 1))
+
+    with self.assertRaisesRegex(
+        AssertionError,
+        _get_err_regex(
+            r'Tree leaf \'x/y\'.*different from expected: \(2, 1\) != \(1, 1\)')
+    ):
+      asserts.assert_tree_shape_suffix(tree, (1, 1))
+
+  def test_assert_tree_shape_suffix_long_suffix(self):
+    tree = {'x': {'y': np.zeros([4, 2, 1])}, 'z': np.zeros([4, 2, 1])}
+    asserts.assert_tree_shape_suffix(tree, (4, 2, 1))
+
+    with self.assertRaisesRegex(
+        AssertionError, _get_err_regex('which is smaller than the expected')):
+      asserts.assert_tree_shape_suffix(tree, (3, 4, 2, 1))
+
+  def test_assert_tree_shape_suffix_none(self):
+    tree = {'x': np.zeros([3]), 'n': None}
+    asserts.assert_tree_shape_suffix(tree, (3,), ignore_nones=True)
+
+    with self.assertRaisesRegex(AssertionError,
+                                _get_err_regex('`None` detected')):
+      asserts.assert_tree_shape_suffix(tree, (3,), ignore_nones=False)
+
+  def test_assert_trees_all_equal_dtypes(self):
+    t_0 = {'x': np.zeros(3, dtype=np.int16), 'y': np.ones(2, dtype=np.float32)}
+    t_1 = {'x': np.zeros(5, dtype=np.uint16), 'y': np.ones(4, dtype=np.float32)}
+
+    with self.assertRaisesRegex(AssertionError,
+                                _get_err_regex('Trees 0 and 1 differ')):
+      asserts.assert_trees_all_equal_dtypes(t_0, t_1)
+
+    t_2 = {'x': np.zeros(6, dtype=jnp.int16), 'y': np.ones(6, dtype=np.float32)}
+    asserts.assert_trees_all_equal_dtypes(t_0, t_2, t_0)
+
+    with self.assertRaisesRegex(AssertionError,
+                                _get_err_regex('Trees 0 and 4 differ')):
+      asserts.assert_trees_all_equal_dtypes(t_0, t_0, t_2, t_0, t_1, t_2)
+
+    # np vs jnp
+    t_3 = {'x': np.zeros(1, dtype=np.int16), 'y': np.ones(2, dtype=jnp.float32)}
+    t_4 = {
+        'x': np.zeros(1, dtype=jnp.int16),
+        'y': np.ones(2, dtype=jnp.float32)
+    }
+    asserts.assert_trees_all_equal_dtypes(t_0, t_2, t_3, t_4)
+
+    # bfloat16
+    t_5 = {'y': np.ones(2, dtype=np.float16)}
+    t_6 = {'y': np.ones(2, dtype=jnp.bfloat16)}
+    asserts.assert_trees_all_equal_dtypes(t_6, t_6)
+    with self.assertRaisesRegex(AssertionError,
+                                _get_err_regex('Trees 0 and 1 differ')):
+      asserts.assert_trees_all_equal_dtypes(t_5, t_6)
+
+  def test_assert_trees_all_equal_wrong_usage(self):
+    # not an array
+    with self.assertRaisesRegex(AssertionError,
+                                _get_err_regex(r'is not a \(j-\)np array')):
+      asserts.assert_trees_all_equal_dtypes({'x': 1.}, {'x': np.array(1.)})
+
+    # 1 tree
+    with self.assertRaisesRegex(  # pylint:disable=g-error-prone-assert-raises
+        ValueError, 'Assertions over only one tree does not make sense'):
+      asserts.assert_trees_all_equal_dtypes({'x': 1.})
+
+  def test_assert_trees_all_equal_none(self):
+    t_0 = {'x': None, 'y': np.array(2, dtype=np.int32)}
+    t_1 = {'x': None, 'y': np.array([23], dtype=np.int32)}
+    t_2 = {'x': None, 'y': np.array(3, dtype=np.float32)}
+    t_3 = {'y': np.array([23], dtype=np.int32)}
+    with self.assertRaisesRegex(AssertionError,
+                                _get_err_regex('Trees 0 and 2 differ')):
+      asserts.assert_trees_all_equal_dtypes(t_0, t_1, t_2, ignore_nones=True)
+    asserts.assert_trees_all_equal_dtypes(t_0, t_1, ignore_nones=True)
+    with self.assertRaisesRegex(AssertionError,
+                                _get_err_regex('`None` detected')):
+      asserts.assert_trees_all_equal_dtypes(t_0, t_1, ignore_nones=False)
+
+    with self.assertRaisesRegex(AssertionError,
+                                _get_err_regex('trees 0 and 1 do not match')):
+      asserts.assert_trees_all_equal_dtypes(t_0, t_3, ignore_nones=True)
+    with self.assertRaisesRegex(AssertionError,
+                                _get_err_regex('trees 0 and 1 do not match')):
+      asserts.assert_trees_all_equal_dtypes(t_0, t_3, ignore_nones=False)
+
 
 class DevicesAssertTest(parameterized.TestCase):
 
@@ -820,7 +1353,8 @@ class DevicesAssertTest(parameterized.TestCase):
         asserts.assert_devices_available(
             n + 1, devtype, backend=devtype, not_less_than=True)
     else:
-      with self.assertRaisesRegex(RuntimeError, 'Unknown backend'):  # pylint: disable=g-error-prone-assert-raises
+      with self.assertRaisesRegex(RuntimeError,  # pylint: disable=g-error-prone-assert-raises
+                                  '(failed to initialize)|(Unknown backend)'):
         asserts.assert_devices_available(
             n - 1, devtype, backend=devtype, not_less_than=True)
 
