@@ -15,18 +15,19 @@
 """JAX/dm-tree friendly dataclass implementation reusing Python dataclasses."""
 
 import collections
+import dataclasses
 import functools
 
 from absl import logging
-import dataclasses
 import jax
 
+
 FrozenInstanceError = dataclasses.FrozenInstanceError
 _RESERVED_DCLS_FIELD_NAMES = frozenset(("from_tuple", "replace", "to_tuple"))
 
 
-def mappable_dataclass(cls, restricted_inheritance=True):
-  """Exposes dataclass as `collections.abc.Mapping` descendent.
+def mappable_dataclass(cls):
+  """Exposes dataclass as ``collections.abc.Mapping`` descendent.
 
   Allows to traverse dataclasses in methods from `dm-tree` library.
 
@@ -34,22 +35,13 @@ def mappable_dataclass(cls, restricted_inheritance=True):
   (i.e. positional args aren't supported; however can use generators/iterables).
 
   Args:
-    cls: dataclass to mutate.
-    restricted_inheritance: ensure dataclass inherits from `object` or from
-      another `chex.dataclass`.
+    cls: A dataclass to mutate.
 
   Returns:
-    Mutated dataclass implementing `collections.abc.Mapping` interface.
+    Mutated dataclass implementing ``collections.abc.Mapping`` interface.
   """
   if not dataclasses.is_dataclass(cls):
-    raise ValueError(f"Expected dataclass, got {cls} (change wrappers order?)")
-
-  is_dataclass_base = all(map(dataclasses.is_dataclass, cls.__bases__))
-  if (restricted_inheritance and (cls.__bases__ !=
-                                  (object,) and not is_dataclass_base)):
-    raise ValueError(
-        f"Not a pure dataclass: undefined behaviour (bases: {cls.__bases__})."
-        "Disable `restricted_inheritance` to suppress this check.")
+    raise ValueError(f"Expected dataclass, got {cls} (change wrappers order?).")
 
   # Define methods for compatibility with `collections.abc.Mapping`.
   setattr(cls, "__getitem__", lambda self, x: self.__dict__[x])
@@ -61,7 +53,7 @@ def mappable_dataclass(cls, restricted_inheritance=True):
   all_fields = set(f.name for f in cls.__dataclass_fields__.values())
   init_fields = [f.name for f in cls.__dataclass_fields__.values() if f.init]
 
-  @functools.wraps(cls.__init__)
+  @functools.wraps(orig_init)
   def new_init(self, *orig_args, **orig_kwargs):
     if (orig_args and orig_kwargs) or len(orig_args) > 1:
       raise ValueError(
@@ -78,10 +70,13 @@ def mappable_dataclass(cls, restricted_inheritance=True):
 
   cls.__init__ = new_init
 
-  # Update base class.
+  # Update base class to derive from Mapping
   dct = dict(cls.__dict__)
   if "__dict__" in dct:
     dct.pop("__dict__")  # Avoid self-references.
+
+  # Remove object from the sequence of base classes. Deriving from both Mapping
+  # and object will cause a failure to create a MRO for the updated class
   bases = tuple(b for b in cls.__bases__ if b != object)
   cls = type(cls.__name__, bases + (collections.abc.Mapping,), dct)
   return cls
@@ -97,23 +92,44 @@ def dataclass(
     unsafe_hash=False,
     frozen=False,
     mappable_dataclass=True,  # pylint: disable=redefined-outer-name
-    restricted_inheritance=True,
 ):
-  """JAX-friendly wrapper for dataclasses.dataclass."""
+  """JAX-friendly wrapper for :py:func:`dataclasses.dataclass`.
+
+  This wrapper class registers new dataclasses with JAX so that tree utils
+  operate correctly. Additionally a replace method is provided making it easy
+  to operate on the class when made immutable (frozen=True).
+
+  Args:
+    cls: A class to decorate.
+    init: See :py:func:`dataclasses.dataclass`.
+    repr: See :py:func:`dataclasses.dataclass`.
+    eq: See :py:func:`dataclasses.dataclass`.
+    order: See :py:func:`dataclasses.dataclass`.
+    unsafe_hash: See :py:func:`dataclasses.dataclass`.
+    frozen: See :py:func:`dataclasses.dataclass`.
+    mappable_dataclass: If True (the default), methods to make the class
+      implement the :py:class:`collections.abc.Mapping` interface will be
+      generated and the class will include :py:class:`collections.abc.Mapping`
+      in its base classes.
+      `True` is the default, because being an instance of `Mapping` makes
+      `chex.dataclass` compatible with e.g. `jax.tree_*` methods, the `tree`
+      library, or methods related to tensorflow/python/utils/nest.py.
+      As a side-effect, e.g. `np.testing.assert_array_equal` will only check
+      the field names are equal and not the content. Use `chex.assert_tree_*`
+      instead.
+
+  Returns:
+    A JAX-friendly dataclass.
+  """
   dcls = _Dataclass(init, repr, eq, order, unsafe_hash, frozen,
-                    mappable_dataclass, restricted_inheritance)
+                    mappable_dataclass)
   if cls is None:
     return dcls
   return dcls(cls)
 
 
 class _Dataclass():
-  """JAX-friendly wrapper for dataclasses.dataclass.
-
-  This wrapper class registers new dataclasses with JAX so that tree utils
-  operate correctly. Additionally a replace method is provided making it easy
-  to operate on the class when made immutable (frozen=True).
-  """
+  """JAX-friendly wrapper for `dataclasses.dataclass`."""
 
   def __init__(
       self,
@@ -124,7 +140,6 @@ class _Dataclass():
       unsafe_hash=False,
       frozen=False,
       mappable_dataclass=True,  # pylint: disable=redefined-outer-name
-      restricted_inheritance=True,
   ):
     self.init = init
     self.repr = repr  # pylint: disable=redefined-builtin
@@ -133,7 +148,7 @@ class _Dataclass():
     self.unsafe_hash = unsafe_hash
     self.frozen = frozen
     self.mappable_dataclass = mappable_dataclass
-    self.restricted_inheritance = restricted_inheritance
+    self.registered = False
 
   def __call__(self, cls):
     """Forwards class to dataclasses's wrapper and registers it with JAX."""
@@ -162,8 +177,8 @@ class _Dataclass():
                        f"{invalid_fields} ({dcls}).")
 
     if self.mappable_dataclass:
-      dcls = mappable_dataclass(dcls, self.restricted_inheritance)
-      # We remove `collection.abc.Mappable` mixin methods here to allow
+      dcls = mappable_dataclass(dcls)
+      # We remove `collection.abc.Mapping` mixin methods here to allow
       # fields with these names.
       for attr in ("values", "keys", "get", "items"):
         setattr(dcls, attr, None)  # redefine
@@ -181,16 +196,33 @@ class _Dataclass():
     def _getstate(self):
       return self.__dict__
 
+    class_self = self
+
+    # Patch __setstate__ to register the object on deserialization.
     def _setstate(self, state):
+      if not class_self.registered:
+        _register_dataclass_type(dcls)
+        class_self.registered = True
       self.__dict__.update(state)
 
+    orig_init = dcls.__init__
+
+    # Patch object's __init__ such that the class is registered on creation if
+    # it is not registered on deserialization.
+    @functools.wraps(orig_init)
+    def _init(self, *args, **kwargs):
+      if not class_self.registered:
+        _register_dataclass_type(dcls)
+        class_self.registered = True
+      return orig_init(self, *args, **kwargs)
+
     setattr(dcls, "from_tuple", _from_tuple)
     setattr(dcls, "to_tuple", _to_tuple)
     setattr(dcls, "replace", _replace)
     setattr(dcls, "__getstate__", _getstate)
     setattr(dcls, "__setstate__", _setstate)
+    setattr(dcls, "__init__", _init)
 
-    _register_dataclass_type(dcls)
     return dcls
 
 
